#This code is meant to be used as a template for new reactor code. Copy this
#code into a new file in the same folder (or overwrite this file), and
#write your reactor code in the file.

from reactor_code_framework.reactor_manager.reactor_manager import ReactorManager

#Import the classes of the desired hardware and software here! It is important 
#that the classes are in a python file that is within the reactor code folder 
#(or a subfolder in this folder). You can import using the following format,
#or you can comment/uncomment the imports in the comments below.

#from reactor_code_framework.sub_folder_name.file_name import ClassName

#Choose one of the relay classes, depending on the type of relay you use.
#from reactor_code_framework.relay.pi_GPIO_relay import PiGPIORelay
#from reactor_code_framework.relay.blue_GPIO_relay import BlueGPIORelay
from reactor_code_framework.relay.sequent_stackable_relay import SequentStackableRelay

#The following relay-based classes are available.
from reactor_code_framework.relay_based_hardware.lights import Lights
from reactor_code_framework.relay_based_hardware.stir_plate import StirPlate
#from reactor_code_framework.relay_based_hardware.rotating_platform import RotatingPlatform
#from reactor_code_framework.relay_based_hardware.gas_solenoid_valve import GasSolenoidValve

#The following analog-to-digital converter (ADC) classes are available.
#from reactor_code_framework.analog_to_digital_converter.adafruit_ads1x15_adc import AdafruitADS1015
#from reactor_code_framework.analog_to_digital_converter.adafruit_ads1x15_adc import AdafruitADS1115

#The following sensor classes are available.
from reactor_code_framework.sensor.current_sensor import CurrentSensor
#from reactor_code_framework.sensor.oxygen_sensor import OxygenSensor
#from reactor_code_framework.sensor.pressure_sensor import PressureSensor

#The following camera classes are available.
#from reactor_code_framework.camera.raspberry_pi_camera import RaspberryPiCamera
from reactor_code_framework.camera.multi_camera import MultiCamera
#from reactor_code_framework.camera.usb_camera import USBCamera

#The software class ReactorScheduler.
from reactor_code_framework.reactor_scheduler.reactor_scheduler import ReactorScheduler

#NOTE: import any other modules here.

#Create a reactor manager. The variables passed in to the reactor_manager
#should not be modified unless you know what you are doing. The first is the
#mode for the pins on the raspberry pi (either "BCM" or "BOARD"). The second
#is the absolute free disk space limit for the program (if the free disk space
#drops below this limit, the program will crash). The third is the startup
#free disk space limit, and if the free disk space is below this limit at the
#start of the program, the program will crash. The fourth input is the relative
#file path of the config file.
#NOTE: ALWAYS create the reactor manager AFTER all imports but BEFORE 
#creating any of the instances of the hardware.

reactor_manager = ReactorManager("BCM", 5.0, 6.5, "config.yaml")

#You can use the reactor_manager.read_config_variables method to update the 
#reactor config variables at any point in your run function.
config_variables = reactor_manager.read_config_variables()

#Add the hardware here. For each hardware instance created, it must be
#added to the hardware manager. Use the code below as a template:

#device = DeviceClassName(reactor_manager, other_inputs)

#NOTE: The software elements should be created within the code itself.

#Write the code for the reactor in this function.
def run():
  #Write any code for the startup of the reactor here.

  #Write any variables that need to be used by the following subroutines.

  #Define any subroutines for repetive tasks (taking pictures, reading sensors,
  #degassing, etc.). If necessary, you can utilize variables defined at the
  #start of the run function. The "nonlocal" keyword must be used in the 
  #subroutines to access these variables. These variables act kind of like 
  #global variables within the run function.

  #Wrap these task functions in Event or SequenceEvent.

  #Write loops for cycles if necessary.

  #Create instances of the ReactorScheduler class to handle the repetitive 
  #events.
  #NOTE: these instances may need to be inside the loops for your cycles (if
  #your reactor has cycles).
  #for example:
  #descriptive_name_scheduler = ReactorScheduler(reactor_manager, total_time = TOTAL TIME TO LOOP)
  #descriptive_name_scheduler.add_event(event)
  #descriptive_name_scheduler.loop()
  return

#This tries to run the reactor code, then runs the appropriate cleanup code
#after the code has finished executing or after an error.
try:
  run()
except:
  print("Error Encountered!")
  reactor_manager.print_exception()
finally:
  print("Cleaning up...")
  reactor_manager.cleanup()
  print("Exiting...")