import os
import yaml

#The purpose of this class is to handle the creation of folders, and the
#reading and writing of text files.
class IOManager(object):
  #Store an instance of ReactorManager as reactor_manager.
  def __init__(self, reactor_manager):
    self.reactor_manager = reactor_manager
  
  #Represent the IOManager as a string.
  def __repr__(self):
    return "IOManager"

  #Given a directory path, dir_path, this function makes the directory as
  #specified, if it doesn't exist. 
  def make_directory(self, dir_path):
    if not os.path.isdir(dir_path):
      os.makedirs(dir_path)
    else:
      print("Redundant make_directory call: Directory {0} already exists.".format(dir_path))

  #write_text_file and read_text_file are modified from 15-112:
  #http://www.krivers.net/15112-s19/notes/notes-strings.html

  #Given a file path, path, ending in .txt, read the contents of the file and 
  #return the contents as a string.
  def read_text_file(self, path):
    with open(path, "rt") as f:
        return f.read()
  
  #Given a file path, path, and a string, contents, write the string contents
  #into a file at the given file path.
  def write_text_file(self, path, contents):
    with open(path, "wt") as f:
        f.write(contents)
  
  #Given a text file path, path, append the content contained in
  #additional_content to the text file. Note, if the file does not yet exist,
  #it will be created by this function.
  def write_append_text_file(self, path, additional_content):
    with open(path, "at") as f:
      f.write(additional_content)

  #Given a relative file path, return the absolute file path.
  def get_abs_path(self, relative_path):
    return os.path.abspath(relative_path)

  #Given the path from a file, return the directory as a string.
  def get_directory_from_path(self, path):
    return os.path.dirname(path)

  #Given a file in YAML format, read the file into python as a dictionary.
  #Please see the example of YAML format in the examples folder in this project.
  #For more information, see:
  #https://www.cloudbees.com/blog/yaml-tutorial-everything-you-need-get-started
  #https://pyyaml.org/wiki/PyYAMLDocumentation
  def read_parameters_from_yaml(self, yaml_file_path):
    return yaml.safe_load(self.read_text_file(yaml_file_path))