from reactor_code_framework.relay.generic_relay import GenericRelay

import RPi.GPIO as GPIO

#This class is a catch-all class intended to be used with the non-pi,
#non-"sequent stackable", GPIO controlled, generally blue-colored relays.
#This functionality is similar to the pi_GPIO_relay, except GPIO.high and
#GPIO.low are flipped in set_relay_state.
class BlueGPIORelay(GenericRelay):
  #Store the GPIO pin and run GPIO setup on top of the generic setup.
  def __init__(self, reactor_manager, GPIO_Pin):
    self.GPIO_pin = GPIO_Pin
    GPIO.setup(self.GPIO_pin, GPIO.OUT)
    super().__init__(reactor_manager, "BlueGPIORelay", self.GPIO_pin)
  
  #Check that the GPIO pin value is an integer.
  def check_parameters(self):
    if not isinstance(self.GPIO_pin, int):
      raise Exception("Relay Error({0}): GPIO pin {1} {2} not valid.".format(self, self.reactor_manager.get_GPIO_mode(), self.GPIO_pin))

  #Return a string of the type, gpio pin, and state of the relay.
  def __repr__(self):
    return "{0}: GPIO Pin: {1} {2}  State: {3}".format(self.relay_type, self.reactor_manager.get_GPIO_mode(), self.GPIO_pin, self.state)

  #Set the relay to the appropriate state to open or close the relay.
  def set_relay_state(self, state):
    if state == "open":
      GPIO.output(self.GPIO_pin, GPIO.HIGH)
    elif state == "close":
      GPIO.output(self.GPIO_pin, GPIO.LOW)