from reactor_code_framework.relay.generic_relay import GenericRelay

import RPi.GPIO as GPIO

#This class is intended to be used with the Raspberry Pi Relays, which have 
#three relays on the board and interact with GPIO pins. Each relay is
#controlled by specific GPIO pins. The valid pins for these relays are
#26, 20, and 21.
class PiGPIORelay(GenericRelay):
  #Store the GPIO pin and run GPIO setup on top of the generic setup.
  def __init__(self, reactor_manager, GPIO_Pin):
    self.GPIO_pin = GPIO_Pin
    GPIO.setup(self.GPIO_pin, GPIO.OUT)
    super().__init__(reactor_manager, "PiGPIORelay", self.GPIO_pin)
  
  #Check that the GPIO pin value is an integer of the correct value (either
  #26, 20, or 21, since those are the pins that the relays connect to.).
  def check_parameters(self):
    if not (isinstance(self.GPIO_pin, int) and self.GPIO_pin in [26, 20, 21]):
      raise Exception("Relay Error({0}): GPIO pin {1} {2} not valid.".format(self, self.reactor_manager.get_GPIO_mode(), self.GPIO_pin))

  #Return a string of the type, gpio pin, and state of the relay.
  def __repr__(self):
    return "{0}: GPIO Pin: {1} {2}  State: {3}".format(self.relay_type, self.reactor_manager.get_GPIO_mode(), self.GPIO_pin, self.state)

  #Set the relay to the appropriate state to open or close the relay.
  def set_relay_state(self, state):
    if state == "open":
      GPIO.output(self.GPIO_pin, GPIO.LOW)
    elif state == "close":
      GPIO.output(self.GPIO_pin, GPIO.HIGH)