from reactor_code_framework.relay.generic_relay import GenericRelay

import lib8relind

#This class is intended to be used with the Sequent 8-Layer Stackable 8-Relay
#relay board. This does not use GPIO pins. Instead, the lib8relind library
#from https://github.com/SequentMicrosystems/8relind-rpi/tree/main/python is
#used (this must be installed prior to use). Unlike the GPIO relays, these
#relays have a specific stack and numeric position (see below).
class SequentStackableRelay(GenericRelay):
  #Store the stack and relay positions of the relay. Then, call super() for the
  #generic relay setup (no other relay specific setup is required).
  #stack_num is 0-indexed, while relay_num is 1-indexed.
  def __init__(self, reactor_manager, stack_num, relay_num):
    self.stack_num = stack_num
    self.relay_num = relay_num
    super().__init__(reactor_manager, "SequentStackableRelay", 
                     (self.stack_num, self.relay_num))

  #Check that the stack and relay values are valid. The stack value should be
  #between 0 and 7 (inclusive on both ends), and the relay value should be
  #between 1 and 8 (inclusive on both ends).
  def check_parameters(self):
    if not (isinstance(self.stack_num, int) and 0 <= self.stack_num <= 7):
      raise Exception("Relay Error({0}): stack number {1} not valid.".format(self, self.stack_num))
    elif not (isinstance(self.relay_num, int) and 1 <= self.relay_num <= 8):
      raise Exception("Relay Error({0}): relay number {1} not valid.".format(self, self.relay_num))

  #Return a string of the type, stack, relay number, and state.
  def __repr__(self):
    return "{0}: Stack Number: {1}  Relay Number: {2}  State: {3}".format(self.relay_type, self.stack_num, self.relay_num, self.state)

  #This function uses the lib8relind library to open and close the relay.
  def set_relay_state(self, state):
    if state == "open":
      lib8relind.set(self.stack_num, self.relay_num, 0)
    elif state == "close":
      lib8relind.set(self.stack_num, self.relay_num, 1)