#This is a generic class for all hardware that rely on a single relay being
#turned on and off. This class is generic and not intended for use. Please
#use a subclass of this class.
class GenericRelayBasedHardware(object):
  #Take a ReactorManager instance and a string for the device type. Store both.
  #Take in an instance of any subclass of GenericRelay as relay. Store this
  #relay, check that it is an instance of one of the subclasses of GenericRelay,
  #and turn off the device off to start.
  def __init__(self, reactor_manager, device_type, relay):
    self.reactor_manager = reactor_manager
    self.device_type = device_type
    self.relay = relay
    self.state = None
    self.off()
    self.check_parameters()
    #Pass the instance for the device to the reactor manager.
    self.reactor_manager.add_hardware_device(self)

  #Check that the provided relay instance is valid by checking that it has open 
  #and close methods. This does not check the actual hardware.
  def check_parameters(self):
    if not ((hasattr(self.relay, "open") and callable(self.relay.open)) and \
            (hasattr(self.relay, "close") and callable(self.relay.close))):
      raise Exception("{0} Error({1}): Relay {2} not valid.".format(self.device_type, self, self.relay))

  #Represent the device as a string.
  def __repr__(self):
    return "{0}: Relay {1}".format(self.device_type, self.relay)
  
  #Compare two instances of devices based on their device types, relays, and
  #states.
  def __eq__(self, other):
    return self.device_type == other.device_type and \
           self.relay == other.relay and \
           self.state == other.state
  
  #Hash the device based on the device_type and provided relay.
  def __hash__(self):
    return hash(self.device_type, self.relay)

  #Turn on the device.
  def on(self):
    self.relay.close()
    self.state = "on"

  #Turn off the device.
  def off(self):
    self.relay.open()
    self.state = "off"
  
  #Return the state of the device.
  def get_state(self):
    return self.state
  
  #Cleanup the device by turning it off.
  def cleanup(self):
    self.off()